#include "nds.hh"

#include <memory>
#include <cassert>
#include <cstdlib>
#include <iostream>

#include "nds_testing.hh"

NDS::epochs_type
expected_epochs(NDS::connection::protocol_type proto)
{
	NDS::epochs_type exp;

	if (proto == NDS::connection::PROTOCOL_ONE)
	{
		exp.push_back(NDS::epoch("ALL", 0, NDS::buffer::GPS_INF));
	}
	else
	{
		exp.push_back(NDS::epoch("ALL", 0, 1999999999));
		exp.push_back(NDS::epoch("ER2", 1025636416, 1028563232));
		exp.push_back(NDS::epoch("ER3", 1042934416, 1045353616));
		exp.push_back(NDS::epoch("ER4", 1057881616, 1061856016));
		exp.push_back(NDS::epoch("ER5", 1073606416, 1078790416));
		exp.push_back(NDS::epoch("ER6", 1102089216, 1102863616));
		exp.push_back(NDS::epoch("ER7", 1116700672, 1118330880));
		exp.push_back(NDS::epoch("ER8", 1123856384, 1126621184));
		exp.push_back(NDS::epoch("NONE", 0, 0));
		exp.push_back(NDS::epoch("O1", 1126621184, 1999999999));
		exp.push_back(NDS::epoch("S5", 815153408, 880920032));
		exp.push_back(NDS::epoch("S6", 930960015, 971654415));
		exp.push_back(NDS::epoch("S6a", 930960015, 935798415));
		exp.push_back(NDS::epoch("S6b", 937785615, 947203215));
		exp.push_back(NDS::epoch("S6c", 947635215, 961545615));
		exp.push_back(NDS::epoch("S6d", 961545615, 971654415));
	}
	return exp;
}

int
main(int argc, char *argv[])
{
  using NDS::connection;

  //---------------------------------------------------------------------
  // Obtain the port of the server
  //---------------------------------------------------------------------
  connection::port_type	port = 31200;
  if ( ::getenv("NDS_TEST_PORT") ) 
  {
    std::istringstream	ps( ::getenv("NDS_TEST_PORT") );

    ps >> port;
  }
  std::string hostname("localhost");
  if ( ::getenv("NDS_TEST_HOST") ) 
  {
  	hostname = ::getenv( "NDS_TEST_HOST" );
  }

  connection::protocol_type proto = connection::PROTOCOL_TRY;
  if ( ::getenv("NDS_TEST_PROTO") )
  {
  	std::string ps( ::getenv("NDS_TEST_PROTO") );
    std::cerr << "NDS_TEST_PROTO = " << ps << std::endl;
  	if ( ps.compare("1") == 0 ) proto = connection::PROTOCOL_ONE;
  	else if ( ps.compare("2") == 0 ) proto = connection::PROTOCOL_TWO;
  }

  std::cerr << "Connecting to " << hostname << ":" << port << " " << proto << std::endl;
  pointer< connection > conn( new connection( hostname, port, proto ));

  {
    NDS::epoch cur_epoch = conn->current_epoch();
    assert( cur_epoch.gps_start == 0 );
    assert( cur_epoch.gps_stop == NDS::buffer::GPS_INF );
  }

  NDS::epochs_type epochs = conn->get_epochs();

  NDS::epochs_type ref = expected_epochs( conn->get_protocol() );

  std::cout << "got " << epochs.size() << " entries, expecting " << ref.size() << std::endl;

  for (int i = 0; i < epochs.size(); ++i) 
  {
  	std::cout << epochs[i].name << "=" << epochs[i].gps_start << "-" << epochs[i].gps_stop << std::endl;
  }

  assert( epochs.size() == ref.size() );
  for (int i = 0; i < epochs.size(); ++i) 
  {
  	assert( epochs[i].name == ref[i].name );
  	assert( epochs[i].gps_start == ref[i].gps_start );
  	assert( epochs[i].gps_stop == ref[i].gps_stop );
  }
  std::cout << "Epochs verified" << std::endl;

  if ( conn->get_protocol() == connection::PROTOCOL_TWO )
  {
    for (int i = 0; i < epochs.size(); ++i)
    {
      if (epochs[i].name=="NONE") continue;
      std::cout << "Setting epoch " << epochs[i].name << std::endl;
      conn->set_epoch( epochs[i].name );

      NDS::epoch cur_epoch = conn->current_epoch();
      assert( cur_epoch.gps_start == epochs[i].gps_start );
      assert( cur_epoch.gps_stop == epochs[i].gps_stop );
    }

    for (int i = 0; i < epochs.size(); ++i)
    {
      if (epochs[i].gps_stop == 0) continue;
      std::cout << "Setting epoch " << epochs[i].gps_start << "-" << epochs[i].gps_stop << std::endl;
      conn->set_epoch( epochs[i].gps_start, epochs[i].gps_stop );

      NDS::epoch cur_epoch = conn->current_epoch();
      assert( cur_epoch.gps_start == epochs[i].gps_start );
      assert( cur_epoch.gps_stop == epochs[i].gps_stop );
    }
  }
  return 0;
}
